// Generated by CoffeeScript 1.10.0
var Reader, Utility, iconv,
  extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
  hasProp = {}.hasOwnProperty;

Utility = (require('./utility')).Utility;

iconv = require('iconv-lite');

Reader = (function(superClass) {
  extend(Reader, superClass);

  function Reader() {
    return Reader.__super__.constructor.apply(this, arguments);
  }

  Reader.prototype.encode = 'shift_jis';

  Reader.prototype.headerByte = 50;

  Reader.prototype.boneByte = 111;

  Reader.prototype.morphByte = 23;

  Reader.prototype.cameraByte = 61;

  Reader.prototype.lightByte = 28;

  Reader.prototype.shadowByte = 9;

  Reader.prototype.readMeta = function(buffer) {
    var begin, bone, camera, header, ik, light, morph, shadow;
    header = {};
    header.begin = 0;
    header.byte = this.headerByte;
    header.signature = 30;
    header.name = 20;
    header.total = 50;
    begin = header.begin + (header.byte * 1);
    bone = {};
    bone.count = buffer.readUInt32LE(begin);
    bone.begin = begin + 4;
    bone.byte = this.boneByte;
    bone.total = bone.byte * bone.count;
    begin = bone.begin + (bone.byte * bone.count);
    morph = {};
    morph.count = buffer.readUInt32LE(begin);
    morph.begin = begin + 4;
    morph.byte = this.morphByte;
    morph.total = morph.byte * morph.count;
    begin = morph.begin + (morph.byte * morph.count);
    camera = {};
    camera.count = buffer.readUInt32LE(begin);
    camera.begin = begin + 4;
    camera.byte = this.cameraByte;
    camera.total = camera.byte * camera.count;
    begin = camera.begin + (camera.byte * camera.count);
    if (buffer.length > begin) {
      light = {};
      light.count = buffer.readUInt32LE(begin);
      light.begin = begin + 4;
      light.byte = this.lightByte;
      light.total = light.byte * light.count;
      if (buffer.length < light.begin + light.total) {
        light = null;
      }
    }
    begin = (light != null ? light.begin : void 0) + ((light != null ? light.byte : void 0) * (light != null ? light.count : void 0));
    if (buffer.length > begin) {
      shadow = {};
      shadow.count = buffer.readUInt32LE(begin);
      shadow.begin = begin + 4;
      shadow.byte = this.shadowByte;
      shadow.total = shadow.byte * shadow.count;
      if (buffer.length < shadow.begin + shadow.total) {
        shadow = null;
      }
    }
    begin = (shadow != null ? shadow.begin : void 0) + ((shadow != null ? shadow.byte : void 0) * (shadow != null ? shadow.count : void 0));
    if (buffer.length > begin) {
      ik = {};
      ik.count = buffer.readUInt32LE(begin);
      ik.begin = begin + 4;
      ik.number = ik.count > 0 ? buffer.readUInt32LE(begin + 4 + 5) : 0;
      ik.byte = 9 + 21 * ik.number;
      ik.total = ik.byte * ik.count;
      if (buffer.length < ik.begin + ik.total) {
        ik = null;
      }
    }
    return {
      header: header,
      bone: bone,
      morph: morph,
      camera: camera,
      light: light,
      shadow: shadow,
      ik: ik
    };
  };

  Reader.prototype.readHeader = function(buffer) {
    var meta, name, signature;
    meta = (this.readMeta(buffer)).header;
    if (meta == null) {
      return [];
    }
    signature = (this.sliceTrim(buffer, 0, meta.signature)).toString();
    name = iconv.decode(this.sliceTrim(buffer, meta.signature, meta.signature + meta.name), 'shift_jis');
    return {
      signature: signature,
      name: name
    };
  };

  Reader.prototype.readBone = function(buffer) {
    var begin, bezier, frame, i, j, l, meta, name, position, quaternion, ref, results;
    meta = (this.readMeta(buffer)).bone;
    if (meta == null) {
      return [];
    }
    results = [];
    for (i = l = 0, ref = meta.count; 0 <= ref ? l < ref : l > ref; i = 0 <= ref ? ++l : --l) {
      begin = meta.begin + (meta.byte * i);
      j = 0;
      name = iconv.decode(this.sliceTrim(buffer, begin, begin + 15), this.encode);
      j += 15;
      frame = buffer.readUInt32LE(begin + j);
      j += 4;
      position = this.readVector3LE(buffer, begin + j);
      j += 12;
      quaternion = this.readVector4LE(buffer, begin + j);
      j += 16;
      bezier = this.getBezier(buffer.slice(begin + j, begin + j + 64));
      j += 64;
      results.push({
        frame: frame,
        name: name,
        position: position,
        quaternion: quaternion,
        bezier: bezier
      });
    }
    return results;
  };

  Reader.prototype.readMorph = function(buffer) {
    var begin, frame, i, j, l, meta, name, ref, results, weight;
    meta = (this.readMeta(buffer)).morph;
    if (meta == null) {
      return [];
    }
    results = [];
    for (i = l = 0, ref = meta.count; 0 <= ref ? l < ref : l > ref; i = 0 <= ref ? ++l : --l) {
      begin = meta.begin + (meta.byte * i);
      j = 0;
      name = iconv.decode(this.sliceTrim(buffer, begin, begin + 15), this.encode);
      j += 15;
      frame = buffer.readUInt32LE(begin + j);
      j += 4;
      weight = buffer.readFloatLE(begin + j);
      j += 4;
      results.push({
        frame: frame,
        name: name,
        weight: weight
      });
    }
    return results;
  };

  Reader.prototype.readCamera = function(buffer) {
    var begin, bezier, frame, i, j, l, length, location, meta, parspective, ref, results, rotation, viewAngle;
    meta = (this.readMeta(buffer)).camera;
    if (meta == null) {
      return [];
    }
    results = [];
    for (i = l = 0, ref = meta.count; 0 <= ref ? l < ref : l > ref; i = 0 <= ref ? ++l : --l) {
      begin = meta.begin + (meta.byte * i);
      j = 0;
      frame = buffer.readUInt32LE(begin + j);
      j += 4;
      length = (buffer.readFloatLE(begin + j)) * -1;
      j += 4;
      location = this.readVector3LE(buffer, begin + j);
      j += 12;
      rotation = this.readVector3LE(buffer, begin + j);
      rotation[0] *= -1;
      j += 12;
      bezier = this.getBezierCamera(buffer.slice(begin + j, begin + j + 24));
      j += 24;
      viewAngle = buffer.readUInt32LE(begin + j);
      j += 4;
      parspective = (buffer.readUIntLE(begin + j)) === 0;
      j += 1;
      results.push({
        frame: frame,
        length: length,
        location: location,
        rotation: rotation,
        bezier: bezier,
        viewAngle: viewAngle,
        parspective: parspective
      });
    }
    return results;
  };

  Reader.prototype.readLight = function(buffer) {
    var begin, frame, i, j, l, location, meta, ref, results, rgb;
    meta = (this.readMeta(buffer)).light;
    if (meta == null) {
      return [];
    }
    results = [];
    for (i = l = 0, ref = meta.count; 0 <= ref ? l < ref : l > ref; i = 0 <= ref ? ++l : --l) {
      begin = meta.begin + (meta.byte * i);
      j = 0;
      frame = buffer.readUInt32LE(begin + j);
      j += 4;
      rgb = this.readVector3LE(buffer, begin + j);
      j += 12;
      location = this.readVector3LE(buffer, begin + j);
      j += 12;
      results.push({
        frame: frame,
        rgb: rgb,
        location: location
      });
    }
    return results;
  };

  Reader.prototype.readShadow = function(buffer) {
    var begin, distance, frame, i, j, l, meta, mode, ref, results;
    meta = (this.readMeta(buffer)).shadow;
    if (meta == null) {
      return [];
    }
    results = [];
    for (i = l = 0, ref = meta.count; 0 <= ref ? l < ref : l > ref; i = 0 <= ref ? ++l : --l) {
      begin = meta.begin + (meta.byte * i);
      j = 0;
      frame = buffer.readUInt32LE(begin + j);
      j += 4;
      mode = buffer.readUInt8(begin + j);
      j += 1;
      distance = buffer.readFloatLE(begin + j);
      j += 4;
      results.push({
        frame: frame,
        mode: mode,
        distance: distance
      });
    }
    return results;
  };

  Reader.prototype.readIK = function(buffer) {
    var begin, count, enable, frame, i, iks, j, k, l, meta, name, ref, results, show;
    meta = (this.readMeta(buffer)).ik;
    if (meta == null) {
      return [];
    }
    results = [];
    for (i = l = 0, ref = meta.count; 0 <= ref ? l < ref : l > ref; i = 0 <= ref ? ++l : --l) {
      begin = meta.begin + (meta.byte * i);
      j = 0;
      frame = buffer.readUInt32LE(begin + j);
      j += 4;
      show = (buffer.readUIntLE(begin + j)) === 1;
      j += 1;
      count = buffer.readUInt32LE(begin + j);
      j += 4;
      iks = (function() {
        var m, ref1, results1;
        results1 = [];
        for (k = m = 0, ref1 = count; 0 <= ref1 ? m < ref1 : m > ref1; k = 0 <= ref1 ? ++m : --m) {
          name = iconv.decode(this.sliceTrim(buffer, begin + j, begin + j + 20), this.encode);
          j += 20;
          enable = (buffer.readUIntLE(begin + j)) === 1;
          j += 1;
          results1.push({
            name: name,
            enable: enable
          });
        }
        return results1;
      }).call(this);
      results.push({
        frame: frame,
        show: show,
        count: count,
        iks: iks
      });
    }
    return results;
  };

  return Reader;

})(Utility);

module.exports = new Reader;

module.exports.Reader = Reader;
